#include <unistd.h>
#ifdef ENABLE_ARTS
#include <artsc/artsc.h>
#endif
#ifdef ENABLE_ESD
#include <esd.h>
#endif
#include <pulse/simple.h>
#include <alsa/asoundlib.h>
#include <stdlib.h>
#include <dlfcn.h>

int main(int argc,char **argv){
  snd_pcm_t *handle;
  int suspended;
  int sock;

  void *libpulse = NULL;
  pa_simple *s;
  pa_sample_spec ss;
  ss.format = PA_SAMPLE_S16NE;
  ss.channels = 2;
  ss.rate = 44100;
  pa_simple* (*_pa_simple_new)(const char *, const char *, pa_stream_direction_t, const char *, const char *, const pa_sample_spec *, const pa_channel_map *, const pa_buffer_attr *, int *) = NULL;
  void (*_pa_simple_free)(pa_simple *) = NULL;

#ifdef ENABLE_ESD
  void *libesd = NULL;
  int (*_esd_open_sound)(const char *) = NULL;
  esd_standby_mode_t (*_esd_get_standby_mode)(int) = NULL;
  int (*_esd_close)(int) = NULL;
#endif

#ifdef ENABLE_ARTS
  void *libartsc = NULL;
  int (*_arts_init)() = NULL;
#ifdef HAVE_ARTS_SUSPENDED
  int (*_arts_suspended)() = NULL;
#endif
  void (*_arts_free)() = NULL;
#endif

  void *libasound = NULL;
  int (*_snd_pcm_open)(snd_pcm_t **, const char *, snd_pcm_stream_t, int) = NULL;
  int (*_snd_pcm_close)(snd_pcm_t *) = NULL;

  int ossp = WEXITSTATUS(system("pidof osspd >/dev/null 2>&1"));

  if (ossp == 0)
    return execvp(argv[1],argv+1);

  libpulse=dlopen("libpulse-simple.so.0", RTLD_LAZY);
  if (libpulse){
    _pa_simple_new=dlsym(libpulse, "pa_simple_new");
    _pa_simple_free=dlsym(libpulse, "pa_simple_free");
    
    if ((s = _pa_simple_new(NULL, argv[1], PA_STREAM_PLAYBACK, NULL, "test", &ss, NULL, NULL, NULL))){
      _pa_simple_free(s);
      dlclose(libpulse);
      argv[0] = "padsp";
      if(execvp("padsp",argv) == 0)
	return 0;}}

  libasound=dlopen("libasound.so.2", RTLD_LAZY);
  if (libasound){
    _snd_pcm_open=dlsym(libasound, "snd_pcm_open");
    _snd_pcm_close=dlsym(libasound, "snd_pcm_close");

    if (_snd_pcm_open(&handle, "default", SND_PCM_STREAM_PLAYBACK, SND_PCM_NONBLOCK) == 0){
      _snd_pcm_close(handle);
      dlclose(libasound);
      argv[0] = "aoss";
      if(execvp("aoss",argv) == 0)
	return 0;}}

#ifdef ENABLE_ARTS
  libartsc=dlopen("libartsc.so.0", RTLD_LAZY);
  if (libartsc){
    _arts_init=dlsym(libartsc, "arts_init");
#ifdef HAVE_ARTS_SUSPENDED
    _arts_suspended=dlsym(libartsc, "arts_suspended");
#endif
    _arts_free=dlsym(libartsc, "arts_free");

#ifdef HAVE_ARTS_SUSPENDED
    if (_arts_init() == 0 && _arts_suspended() == 0){
#else
    if (_arts_init() == 0){
#endif
      _arts_free();
      dlclose(libartsc);
      argv[0] = "artsdsp";
      if(execvp("artsdsp",argv) == 0)
	return 0;}}
#endif

#ifdef ENABLE_ESD
  libesd=dlopen("libesd.so.0", RTLD_LAZY);
  if (libesd){
    _esd_open_sound=dlsym(libesd, "esd_open_sound");
    _esd_get_standby_mode=dlsym(libesd, "esd_get_standby_mode");
    _esd_close=dlsym(libesd,"esd_close");

    /* don't wake up the beast while detecting */
    setenv("ESD_NO_SPAWN", "1", 1);
    if ((sock = _esd_open_sound(NULL)) >= 0){
      if (_esd_get_standby_mode(sock) == ESM_RUNNING){
	_esd_close(sock);
	dlclose(libesd);
	argv[0] = "esddsp";
	if(execvp("esddsp",argv) == 0)
	  return 0;}
      _esd_close(sock);
      dlclose(libesd);}}
#endif

  return execvp(argv[1],argv+1);}
